!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.


!	central_diff.f90- Routine containing routines for the taking the 
!	derivative of a scalar function in one direction.
!	
!	central_diff- 2nd order central diff. scheme, for aribitrary, 
!		cartesian mesh
!
!	tenth_order- 10 order compact scheme for constant dx or dy,
!		cart, and periodic or non-per.
!
!	grad- Takes the gradient of a scalar variable using the compact schemes
!		Used in the solver.f90 (get_tau) to compute gradients
!

 

SUBROUTINE central_diff(f,df,dir,m,n,dx,per)

!	Pass in A in M by N matrix.
!	Take the 2nd order differece of the Matrix A in the direction 
!	(dir) coorespindng to array index
!	Example:  PHI(N,M) is scalar, x(N) = grid spacing, 
!	central_diff(PHI,X,1) = returns B(N,M) of said derivative

IMPLICIT NONE
INTEGER :: dir,m,n,per
DOUBLE PRECISION, DIMENSION(n,m) :: f,df
DOUBLE PRECISION :: dx
INTEGER :: i




IF (dir .eq. 1) THEN

!	dir = 1
DO i = 2, (n-1)
	df(i,:) = (f(i+1,:)-f(i-1,:)) / (2.0D0 * dx)
end DO

IF(per==0) THEN
   !	First order on the edges
   df(1,:) = (f(2,:) - f(1,:))/ dx
   df(n,:) = (f(n,:) - f(n-1,:))/dx
ELSE
   df(1,:) = (f(2,:) - f(n,:))/ (2.0D0*dx)
   df(n,:) = (f(1,:) - f(n-1,:))/ (2.0D0*dx)
END IF

ELSEIF (dir .eq. 2) THEN

!	dir = 2
DO i = 2, (m-1)
	df(:,i) = (f(:,i+1)-f(:,i-1))/(2.0D0*dx)
end DO

IF(per==0) THEN
   !	First order on the edges
   df(:,1) = (f(:,2) - f(:,1))/ dx
   df(:,m) = (f(:,m) - f(:,m-1)) / dx
ELSE
   df(:,1) = (f(:,2) - f(:,m))/ (2.0D0*dx)
   df(:,m) = (f(:,1) - f(:,m-1)) / (2.0D0*dx)
END IF


ELSE 

END IF


END SUBROUTINE central_diff


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


SUBROUTINE tenth_order(f,df,dir,m,n,dx,per)
USE matrices
IMPLICIT NONE
INTEGER :: M,N,mmax,dir,i,j,NN,MM,per
DOUBLE PRECISION, DIMENSION(m,n) :: f,df
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: aa,bb,cc,dd,ee,rhs,u
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: ff,dff
DOUBLE PRECISION :: alpha,beta,a2,b2,c2,dx
DOUBLE PRECISION :: a0,b0,c0,d0,a1,a3,b3,c3
DOUBLE PRECISION :: alpha0, alpha1,alpha2, beta2,alpha3,beta3
DOUBLE PRECISION :: cp1,cp2,cp3,cp4,cp5,cp6
DOUBLE PRECISION :: w0,w1,w2,w3
DOUBLE PRECISION, PARAMETER :: zero = 0.0D0, one = 1.0D0
DOUBLE PRECISION :: even

!  Check 1-d directions
if (m==1 .and. dir==1) then
  df = 0.0d0
  return
end if

if (n==1 .and. dir==2) then
  df = 0.0d0
  return
end if


!	Logic to treat symetric as even or odd
!  EVEN FUNCTION
if (per > 0) even = one

!  ODD FUNCTION
if (per < 0) then
	even = -one
	per = abs(per)
end if


!	Some logic for derivative directions
if (dir .eq. 1) then 
  NN = N
  MM = M
else if (dir .eq. 2) then
  NN = M
  MM = N
end if

ALLOCATE(ff(MM))
ALLOCATE(dff(MM))
ALLOCATE(aa(MM))
ALLOCATE(bb(MM))
ALLOCATE(cc(MM))
ALLOCATE(dd(MM))
ALLOCATE(ee(MM))
ALLOCATE(rhs(MM))
ALLOCATE(u(MM))

!	Constant Coefficients, 10th Order Compact Scheme
alpha = 0.5D0
beta = 0.5D-1
a2 = 17.0D0/12.0D0
b2 = 101.0D0/150.0D0
c2 = 1.0D0/100.0D0

DO j = 1, NN

  	!	Some logic for derivative directions
	if (dir .eq. 1) then 
	  ff(:) = f(:,j)
      dff(:) = df(:,j)
	else if (dir .eq. 2) then
	  ff(:) = f(j,:)
      dff(:) = df(j,:)
	end if

   
!  Periodic only
if (per .eq. 1) then

aa = beta
bb = alpha
cc = 1.0D0
dd = alpha
ee = beta
  
  
rhs(1) = a2*( ff(2)-ff(MM) ) /(2.0D0*dx) + b2*( ff(3)- ff(MM-1) ) / (4.0D0*dx) + &
        c2*( ff(4)-ff(MM-2) )/(6.0D0*dx)
rhs(2) = a2*( ff(3)-ff(1) ) /(2.0D0*dx) + b2*( ff(4)- ff(MM) ) / (4.0D0*dx) + &
        c2*( ff(5)-ff(MM-1) )/(6.0D0*dx)   
rhs(3) = a2*( ff(4)-ff(2) ) /(2.0D0*dx) + b2*( ff(5)- ff(1) ) / (4.0D0*dx) + &
        c2*( ff(6)-ff(MM) )/(6.0D0*dx)

DO i = 4, MM-3  
          rhs(i) = a2*( ff(i+1)-ff(i-1) ) /(2.0D0*dx) + b2*( ff(i+2)- ff(i-2) ) / (4.0D0*dx) + &
  	      c2*( ff(i+3)-ff(i-3) )/(6.0D0*dx)
END DO

rhs(MM-2) = a2*( ff(MM-1)-ff(MM-3) ) /(2.0D0*dx) + b2*( ff(MM)- ff(MM-4) ) / (4.0D0*dx) + &
        c2*( ff(1)-ff(MM-5) )/(6.0D0*dx)
rhs(MM-1) = a2*( ff(MM)-ff(MM-2) ) /(2.0D0*dx) + b2*( ff(1)- ff(MM-3) ) / (4.0D0*dx) + &
        c2*( ff(2)-ff(MM-4) )/(6.0D0*dx)
rhs(MM) = a2*( ff(1)-ff(MM-1) ) /(2.0D0*dx) + b2*( ff(2)- ff(MM-2) ) / (4.0D0*dx) + &
        c2*( ff(3)-ff(MM-3) )/(6.0D0*dx)

!	Corners for Periodic Pent solver
cp1 = beta
cp2 = alpha
cp3 = beta
cp4 = beta
cp5 = alpha
cp6 = beta

CALL cypent(aa,bb,cc,dd,ee,rhs,cp1,cp2,cp3,cp4,cp5,cp6,u,mm)

!	Normal for non-periodic
elseif (per == 0 .or. per == 2 .or. per == 3 .or. per == 4) then

!	3rd Order Conservative Boundary Conditions
     rhs(1) =-B10d(1,1)*(ff(2)-ff(1)) + B10d(3,1)*(ff(3)-ff(2))
     rhs(2) = B10d(3,2)*(ff(3)-ff(1 ))
     rhs(3) = B10d(4,3)*(ff(4 )-ff(2  )) + B10d(5,3)*(ff(5  )-ff(1  ))
     rhs(4) = B10d(5,4)*(ff(5 )-ff(3  )) + B10d(6,4)*(ff(6  )-ff(2  )) &
               & + B10d(7,4)*(ff(7  )-ff(1  )) 
     rhs(5:mm-4) = B10d(B10dcol-2,B10drow)*(ff(6:mm-3)-ff(4:mm-5)) &
                 & + B10d(B10dcol-1,B10drow)*(ff(7:mm-2)-ff(3:mm-6)) &
                 & + B10d(B10dcol  ,B10drow)*(ff(8:mm-1)-ff(2:mm-7)) 
     rhs(mm-3) = B10d(5,4)*(ff(mm-2)-ff(mm-4)) + B10d(6,4)*(ff(mm-1)-ff(mm-5)) &
               & + B10d(7,4)*(ff(mm )-ff(mm-6))
     rhs(mm-2) = B10d(4,3)*(ff(mm-1)-ff(mm-3)) + B10d(5,3)*(ff(mm )-ff(mm-4))
     rhs(mm-1) = B10d(3,2)*(ff(mm  )-ff(mm-2))
     rhs(mm  ) =-B10d(1,1)*(ff(mm  )-ff(mm-1)) + B10d(3,1)*(ff(mm-1)-ff(mm-2))
     rhs = rhs/dx

	!	Set the Penta-diagonal Matrix
	aa = A10d(3,5)
	bb = A10d(4,5)
	cc = A10d(5,5)
	dd = A10d(6,5)
	ee = A10d(7,5)

	!	Modify the LHS diagonal matrixes for the boundarys
	aa(1) = zero
	aa(2) = zero
	aa(3) = A10d(1,3)
	aa(4) = A10d(2,4)
	aa(mm-3) = A10d(6,4)
	aa(mm-2) = A10d(5,3)
	aa(mm-1) = A10d(4,2)
	aa(mm) = A10d(3,1)

	bb(1) = zero
	bb(2) = A10d(1,2)
	bb(3) = A10d(2,3)
	bb(4) = A10d(3,4)
	bb(mm-3) = A10d(5,4)
	bb(mm-2) = A10d(4,3)
	bb(mm-1) = A10d(3,2)
	bb(mm) 	 = A10d(2,1)

	cc(1) = A10d(1,1)
	cc(2) = A10d(2,2)
	cc(3) = A10d(3,3)
	cc(4) = A10d(4,4)
	cc(mm-3) = A10d(4,4)
	cc(mm-2) = A10d(3,3)
	cc(mm-1) = A10d(2,2)
	cc(mm)   = A10d(1,1)


	dd(1) = A10d(2,1)
	dd(2) = A10d(3,2)
	dd(3) = A10d(4,3)
	dd(4) = A10d(5,4)
	dd(mm-3) = A10d(3,4)
	dd(mm-2) = A10d(2,3)
	dd(mm-1) = A10d(1,2)
	dd(mm) = zero

	ee(1) = A10d(3,1)
	ee(2) = A10d(4,2)
	ee(3) = A10d(5,3)
	ee(4) = A10d(6,4)
	ee(mm-3) = A10d(2,4)
	ee(mm-2) = A10d(1,3)
	ee(mm-1) = zero
	ee(mm) = zero



!	Symmetric Boundary Conditions here (intrinsic and high order)	

	if (per == 2 .or. per == 4) then  ! If this side is symmetric, modify its rhs and lhs
	
	!	Right-Hand Side alterations
	rhs(1) = a2*( ff(2)-even*ff(2) ) /(2.0D0*dx) + b2*( ff(3)- even*ff(3) ) / (4.0D0*dx) + &
	        c2*( ff(4)-even*ff(4) )/(6.0D0*dx)
	rhs(2) = a2*( ff(3)-ff(1) ) /(2.0D0*dx) + b2*( ff(4)- even*ff(2) ) / (4.0D0*dx) + &
	        c2*( ff(5)-even*ff(3) )/(6.0D0*dx)   
	rhs(3) = a2*( ff(4)-ff(2) ) /(2.0D0*dx) + b2*( ff(5)- ff(1) ) / (4.0D0*dx) + &
	        c2*( ff(6)-even*ff(MM) )/(6.0D0*dx)
	i = 4  
	rhs(i) = a2*( ff(i+1)-ff(i-1) ) /(2.0D0*dx) + b2*( ff(i+2)- ff(i-2) ) / (4.0D0*dx) + &
		c2*( ff(i+3)-ff(i-3) )/(6.0D0*dx)
		
	!	Left-Hand Side alterations
	aa(1) = zero
	aa(2) = zero
	aa(3) = beta
	aa(4) = beta
	
	bb(1) = zero
	bb(2) = alpha
	bb(3) = alpha
	bb(4) = alpha
	
	cc(1) = one
	cc(2) = one - even*beta
	cc(3) = one
	cc(4) = one
	
	dd(1) = alpha*(one - even)
	dd(2) = alpha
	dd(3) = alpha
	dd(4) = alpha
	
	ee(1) = beta*(one - even)
	ee(2) = beta
	ee(3) = beta
	ee(4) = beta
		
	
	end if
	
	
	if (per == 3 .or. per == 4) then
	
	 
	!	Modify the RHS
	i = MM-3 
	rhs(i) = a2*( ff(i+1)-ff(i-1) ) /(2.0D0*dx) + b2*( ff(i+2)- ff(i-2) ) / (4.0D0*dx) + &
		c2*( ff(i+3)-ff(i-3) )/(6.0D0*dx)
	rhs(MM-2) = a2*( ff(MM-1)-ff(MM-3) ) /(2.0D0*dx) + b2*( ff(MM)- ff(MM-4) ) / (4.0D0*dx) + &
	        c2*( even*ff(MM-1)-ff(MM-5) )/(6.0D0*dx)
	rhs(MM-1) = a2*( ff(MM)-ff(MM-2) ) /(2.0D0*dx) + b2*( even*ff(MM-1)- ff(MM-3) ) / (4.0D0*dx) + &
	        c2*( even*ff(MM-2)-ff(MM-4) )/(6.0D0*dx)
	rhs(MM) = a2*( even*ff(MM-1)-ff(MM-1) ) /(2.0D0*dx) + b2*( even*ff(MM-2)- ff(MM-2) ) / (4.0D0*dx) + &
	        c2*( even*ff(MM-3)-ff(MM-3) )/(6.0D0*dx)
		
	!	Now the LHS
	aa(mm-3) = beta
	aa(mm-2) = beta
	aa(mm-1) = beta
	aa(mm) = beta*(one-even)

	bb(mm-3) = alpha
	bb(mm-2) = alpha
	bb(mm-1) = alpha
	bb(mm) 	 = alpha*(one - even)
	
	cc(mm-3) = one
	cc(mm-2) = one
	cc(mm-1) = one - even*beta
	cc(mm)   = one

	dd(mm-3) = alpha
	dd(mm-2) = alpha
	dd(mm-1) = alpha
	dd(mm) = zero

	ee(mm-3) = beta
	ee(mm-2) = beta
	ee(mm-1) = zero
	ee(mm) = zero

	
	
	end if
	
	
	!	Solve the Pentadiagonal Matrix
	CALL pentdag(aa,bb,cc,dd,ee,rhs,u,mm)
	

end if
    

	!	Reassign to the derivative array
    if (dir .eq. 1) then 
	  df(:,j) = u(:)
	else if (dir .eq. 2) then
      df(j,:) = u(:)
	end if
    
END DO

END SUBROUTINE tenth_order

!!!!!!!!!!!!!!!!!!


SUBROUTINE grad(Q,dQdx,dQdy)
USE globals, only: x,y,zero,dr
USE inputs, only: periodx, periody,nx,ny,dx,dy,sing
!USE metrics
IMPLICIT NONE
DOUBLE PRECISION, DIMENSION(nx,ny), INTENT(IN)  :: Q
DOUBLE PRECISION, DIMENSION(nx,ny), INTENT(OUT) :: dQdx,dQdy
DOUBLE PRECISION :: dumx, dumy
DOUBLE PRECISION, DIMENSION(nx,5) :: tmp

dQdx = zero
dQdy = zero

!	10th order compact
CALL tenth_order(Q,dQdx,1,nx,ny,dx,periodx)
CALL tenth_order(Q,dQdy,2,nx,ny,dy,periody)

!CALL central_diff(Q,dQdE,1,nx,ny,dE,periodx)
!CALL central_diff(Q,dQdN,2,nx,ny,dN,periody)


!  ADD METRIC TERMS HERE FOR THE CALCULATION OF THESE PHYSICAL DERIVATIVES
!dQdx = ( dQdE*dydN - dQdN*dydE)/dxy
!dQdy = (-dQdE*dxdN + dQdN*dxdE)/dxy


END SUBROUTINE grad


SUBROUTINE div(G,F,Qx,Qy,n)
USE globals, only: i,j,zero,dr
USE inputs, only:nx,ny,periodx,periody,dx,dy
!USE metrics
IMPLICIT NONE
INTEGER :: n
DOUBLE PRECISION, DIMENSION(nx,ny,n) :: qx,qy,G,F
DOUBLE PRECISION :: dumx,dumy
DOUBLE PRECISION, DIMENSION(nx,5) :: tmp

qx = zero
qy = zero

!  MAKE MAPPED FLUXES HERE USING METRIC TERMS FOR G' and F'
!DO i = 1, n
!	Gp(:,:,i) = ( G(:,:,i)*dydN - F(:,:,i)*dxdN)
!	Fp(:,:,i) = (-G(:,:,i)*dydE + F(:,:,i)*dxdE)
!END DO

!  MAKE MAPPED FLUXES HERE USING METRIC TERMS FOR G' and F'
!DO i = 1, n
!	Gp(:,:,i) = ( (G(:,:,i)+artEx(:,:,i))*dydN - (F(:,:,i)+artEy(:,:,i))*dxdN)
!	Fp(:,:,i) = (-(G(:,:,i)+artNx(:,:,i))*dydE + (F(:,:,i)+artNy(:,:,i))*dxdE)
!END DO

!Gp = G
!Fp = F

!	10th order compact
!  X-direction Flux
DO i = 1, n
	CALL tenth_order(G(:,:,i),qx(:,:,i),1,nx,ny,dx,periodx)
	!CALL central_diff(Gp(:,:,i),qx(:,:,i),1,nx,ny,dE,periodx)
END DO



!  Y-direction Flux
DO i = 1, n
	CALL tenth_order(F(:,:,i),qy(:,:,i),2,nx,ny,dy,periody)
	!CALL central_diff(Fp(:,:,i),qy(:,:,i),2,nx,ny,dN,periody)
END DO


END SUBROUTINE div




