!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.


!	Filters.f90- File containing the filter routines used in miranda_2d
!	
!	compact_filter- 8th order compact filter (pentadiagonal) used
!		at every sub-iteration of the RK4 integrator to filter the 
!		conserved variables.  Contains periodic flags and calls a per.
!		and non-per. pent. diag solver.
!
!	gauss- Guassian filter called after the AFLES terms are calculated
!		to remove the cusps introduced by the absolute value operator.
!		Also, is per. and non-periodic.


SUBROUTINE compact_filter(f,dir,m,n,per)
USE matrices
IMPLICIT NONE
INTEGER :: M,N,mmax,dir,i,j,mm,nn,per
DOUBLE PRECISION, DIMENSION(m,n) :: f
DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: aa,bb,cc,dd,ee,rhs,u,ff
DOUBLE PRECISION :: alpha,beta,a1,b2,c2,d2,e2,cp1,cp2,cp3,cp4,cp5,cp6
DOUBLE PRECISION, PARAMETER :: zero = 0.0D0, one = 1.0D0

!  Check 1-d directions
if (m==1 .and. dir==1) then
  return
end if

if (n==1 .and. dir==2) then
  return
end if

!	Some logic for derivative directions
if (dir .eq. 1) then 
  NN = N
  MM = M
else if (dir .eq. 2) then
  NN = M
  MM = N
end if


ALLOCATE(aa(MM))
ALLOCATE(bb(MM))
ALLOCATE(cc(MM))
ALLOCATE(dd(MM))
ALLOCATE(ee(MM))
ALLOCATE(rhs(MM))
ALLOCATE(u(MM))
ALLOCATE(ff(MM))

!	Constant Coefficients
alpha = 0.66624
beta = 0.16688
a1 = 0.99965
b2 = 0.66652
c2 = 0.16674
d2 = 4.0D-5
e2 = -5.0D-6


DO j = 1, NN
  
	if (dir .eq. 1) then 
	  ff(:) = f(:,j)
	else if (dir .eq. 2) then
	  ff(:) = f(j,:)
	end if


    
		!  Periodic only
	if (per .eq. 1) then

		aa = beta
		bb = alpha
		cc = 1.0D0
		dd = alpha
		ee = beta
    
  
		rhs(1) = a1*ff(1) + b2*(ff(mm)+ff(2)) + c2*(ff(mm-1)+ff(3)) + &
        	d2*(ff(mm-2)+ff(4)) + e2*(ff(mm-3)+ff(5))
		rhs(2) = a1*ff(2) + b2*(ff(1)+ff(3)) + c2*(ff(mm)+ff(4)) + &
        	d2*(ff(mm-1)+ff(5)) + e2*(ff(mm-2)+ff(6))
		rhs(3) = a1*ff(3) + b2*(ff(2)+ff(4)) + c2*(ff(1)+ff(5)) + &
        	d2*(ff(mm)+ff(6)) + e2*(ff(mm-1)+ff(7))
        rhs(4) = a1*ff(4) + b2*(ff(3)+ff(5)) + c2*(ff(2)+ff(6)) + &
        	d2*(ff(1)+ff(7)) + e2*(ff(mm)+ff(8))

		DO i = 5, MM-4  
    		rhs(i) = a1*ff(i) + b2*(ff(i-1)+ff(i+1)) + c2*(ff(i-2)+ff(i+2)) + &
        	d2*(ff(i-3)+ff(i+3)) + e2*(ff(i-4)+ff(i+4))
  		END DO
            
		rhs(MM-3) = a1*ff(MM-3) + b2*(ff(MM-4)+ff(MM-2)) + c2*(ff(MM-5)+ff(MM-1)) + &
        	d2*(ff(MM-6)+ff(MM)) + e2*(ff(MM-7)+ff(1))
		rhs(MM-2) = a1*ff(MM-2) + b2*(ff(MM-3)+ff(MM-1)) + c2*(ff(MM-4)+ff(MM)) + &
        	d2*(ff(MM-5)+ff(1)) + e2*(ff(MM-6)+ff(2))
		rhs(MM-1) = a1*ff(MM-1) + b2*(ff(MM-2)+ff(MM)) + c2*(ff(MM-3)+ff(1)) + &
        	d2*(ff(MM-4)+ff(2)) + e2*(ff(MM-5)+ff(3))
		rhs(MM)   = a1*ff(MM) + b2*(ff(MM-1)+ff(1)) + c2*(ff(MM-2)+ff(2)) + &
	        d2*(ff(MM-3)+ff(3)) + e2*(ff(MM-4)+ff(4))
            
		!	Corners for Periodic Pent solver
		cp1 = beta
		cp2 = alpha
		cp3 = beta
		cp4 = beta
		cp5 = alpha
        cp6 = beta

		CALL cypent(aa,bb,cc,dd,ee,rhs,cp1,cp2,cp3,cp4,cp5,cp6,u,mm)

		!	Normal for non-periodic
	else


    rhs(1) = B8f(1,1)*ff(1) + B8f(2,1)*ff(2) + B8f(3,1)*ff(3) &
               + B8f(4,1)*ff(4) + B8f(5,1)*ff(5) + B8f(6,1)*ff(6) &
               + B8f(7,1)*ff(7) + B8f(8,1)*ff(8) + B8f(9,1)*ff(9)
    rhs(2) = B8f(1,2)*ff(1) + B8f(2,2)*ff(2) + B8f(3,2)*ff(3) &
               + B8f(4,2)*ff(4) + B8f(5,2)*ff(5) + B8f(6,2)*ff(6) &
               + B8f(7,2)*ff(7) + B8f(8,2)*ff(8) + B8f(9,2)*ff(9)
    rhs(3) = B8f(1,3)*ff(1) + B8f(2,3)*ff(2) + B8f(3,3)*ff(3) &
               + B8f(4,3)*ff(4) + B8f(5,3)*ff(5) + B8f(6,3)*ff(6) &
               + B8f(7,3)*ff(7) + B8f(8,3)*ff(8) + B8f(9,3)*ff(9)
    rhs(4) = B8f(1,4)*ff(1) + B8f(2,4)*ff(2) + B8f(3,4)*ff(3) &
               + B8f(4,4)*ff(4) + B8f(5,4)*ff(5) + B8f(6,4)*ff(6) &
               + B8f(7,4)*ff(7) + B8f(8,4)*ff(8) + B8f(9,4)*ff(9)
    rhs(5:mm-4) = B8f(B8fcol-4,B8frow)* ff(5:mm-4)    &
                   + B8f(B8fcol-3,B8frow)*(ff(4:mm-5) + ff(6:mm-3)) &
                   + B8f(B8fcol-2,B8frow)*(ff(3:mm-6) + ff(7:mm-2)) &
                   + B8f(B8fcol-1,B8frow)*(ff(2:mm-7) + ff(8:mm-1)) &
                   + B8f(B8fcol  ,B8frow)*(ff(1:mm-8) + ff(9:mm  ))
    rhs(mm-3) = B8f(1,4)*ff(mm  ) + B8f(2,4)*ff(mm-1) + B8f(3,4)*ff(mm-2) &
                 + B8f(4,4)*ff(mm-3) + B8f(5,4)*ff(mm-4) + B8f(6,4)*ff(mm-5) &
                 + B8f(7,4)*ff(mm-6) + B8f(8,4)*ff(mm-7) + B8f(9,4)*ff(mm-8)
    rhs(mm-2) = B8f(1,3)*ff(mm  ) + B8f(2,3)*ff(mm-1) + B8f(3,3)*ff(mm-2) &
                 + B8f(4,3)*ff(mm-3) + B8f(5,3)*ff(mm-4) + B8f(6,3)*ff(mm-5) &
                 + B8f(7,3)*ff(mm-6) + B8f(8,3)*ff(mm-7) + B8f(9,3)*ff(mm-8)
    rhs(mm-1) = B8f(1,2)*ff(mm  ) + B8f(2,2)*ff(mm-1) + B8f(3,2)*ff(mm-2) &
                 + B8f(4,2)*ff(mm-3) + B8f(5,2)*ff(mm-4) + B8f(6,2)*ff(mm-5) &
                 + B8f(7,2)*ff(mm-6) + B8f(8,2)*ff(mm-7) + B8f(9,2)*ff(mm-8)
    rhs(mm  ) = B8f(1,1)*ff(mm  ) + B8f(2,1)*ff(mm-1) + B8f(3,1)*ff(mm-2) &
                 + B8f(4,1)*ff(mm-3) + B8f(5,1)*ff(mm-4) + B8f(6,1)*ff(mm-5) &
                 + B8f(7,1)*ff(mm-6) + B8f(8,1)*ff(mm-7) + B8f(9,1)*ff(mm-8)
        

	aa = A8f(3,5)
	bb = A8f(4,5)
	cc = A8f(5,5)
	dd = A8f(6,5)
	ee = A8f(7,5)

	!	Modify the LHS diagonal matrixes for the boundarys
	aa(1) = zero
	aa(2) = zero
	aa(3) = A8f(1,3)
	aa(4) = A8f(2,4)
	aa(mm-3) = A8f(6,4)
	aa(mm-2) = A8f(5,3)
	aa(mm-1) = A8f(4,2)
	aa(mm) = A8f(3,1)

	bb(1) = zero
	bb(2) = A8f(1,2)
	bb(3) = A8f(2,3)
	bb(4) = A8f(3,4)
	bb(mm-3) = A8f(5,4)
	bb(mm-2) = A8f(4,3)
	bb(mm-1) = A8f(3,2)
	bb(mm) 	 = A8f(2,1)

	cc(1) = A8f(1,1)
	cc(2) = A8f(2,2)
	cc(3) = A8f(3,3)
	cc(4) = A8f(4,4)
	cc(mm-3) = A8f(4,4)
	cc(mm-2) = A8f(3,3)
	cc(mm-1) = A8f(2,2)
	cc(mm)   = A8f(1,1)


	dd(1) = A8f(2,1)
	dd(2) = A8f(3,2)
	dd(3) = A8f(4,3)
	dd(4) = A8f(5,4)
	dd(mm-3) = A8f(3,4)
	dd(mm-2) = A8f(2,3)
	dd(mm-1) = A8f(1,2)
	dd(mm) = zero

	ee(1) = A8f(3,1)
	ee(2) = A8f(4,2)
	ee(3) = A8f(5,3)
	ee(4) = A8f(6,4)
	ee(mm-3) = A8f(2,4)
	ee(mm-2) = A8f(1,3)
	ee(mm-1) = zero
	ee(mm) = zero

	CALL pentdag(aa,bb,cc,dd,ee,rhs,u,mm)

	end if

    	!	Reassign to the derivative array
    if (dir .eq. 1) then 
	  f(:,j) = u(:)
	else if (dir .eq. 2) then
      f(j,:) = u(:)
	end if
    
END DO
 

END SUBROUTINE compact_filter

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE gauss(GS)
USE globals
USE inputs, ONLY: nx,ny,periodx,periody
!	Apply gaussian filter as define by Cook, A.W., Phys. Fluid 19 (2007)
DOUBLE PRECISION, DIMENSION(nx,ny) :: GS 
DOUBLE PRECISION :: a1,a2,a3,a4,a5


!	Filter Coefficients
a1 = DBLE(3565.0D0/10368.0D0)
a2 = DBLE(3091.0D0/12960.0D0)
a3 = DBLE(1997.0D0/25920.0D0)
a4 = DBLE(149.0D0/12960.0D0)
a5 = DBLE(107.0D0/103680.0D0)

!	X-direction guassian filter
if (size(x) >= 9) then
DO i=5, size(x)-4

    GS(i,:) = a1*GS(i,:) + a2*(GS(i-1,:)+GS(i+1,:)) + a3*(GS(i-2,:)+GS(i+2,:)) &
    	+ a4*(GS(i-3,:)+GS(i+3,:)) + a5*(GS(i-4,:)+GS(i+4,:))
END DO

if (periodx .eq. 1) then
	GS(1,:) = a1*GS(1,:) + a2*(GS(nx,:)+GS(2,:)) + a3*(GS(nx-1,:)+GS(3,:)) &
    	+ a4*(GS(nx-2,:)+GS(4,:)) + a5*(GS(nx-3,:)+GS(5,:))
    GS(2,:) = a1*GS(2,:) + a2*(GS(1,:)+GS(3,:)) + a3*(GS(nx,:)+GS(4,:)) &
    	+ a4*(GS(nx-1,:)+GS(5,:)) + a5*(GS(nx-2,:)+GS(6,:))
    GS(3,:) = a1*GS(3,:) + a2*(GS(2,:)+GS(4,:)) + a3*(GS(1,:)+GS(5,:)) &
    	+ a4*(GS(nx,:)+GS(6,:)) + a5*(GS(nx-1,:)+GS(7,:))
    GS(4,:) = a1*GS(4,:) + a2*(GS(3,:)+GS(5,:)) + a3*(GS(2,:)+GS(6,:)) &
    	+ a4*(GS(1,:)+GS(7,:)) + a5*(GS(nx,:)+GS(8,:))
    

	GS(nx-3,:) = a1*GS(nx-3,:) + a2*(GS(nx-4,:)+GS(nx-2,:)) + a3*(GS(nx-5,:)+GS(nx-1,:)) &
    	+ a4*(GS(nx-6,:)+GS(nx,:)) + a5*(GS(nx-7,:)+GS(1,:))
    GS(nx-2,:) = a1*GS(nx-2,:) + a2*(GS(nx-3,:)+GS(nx-1,:)) + a3*(GS(nx-4,:)+GS(nx,:)) &
    	+ a4*(GS(nx-5,:)+GS(1,:)) + a5*(GS(nx-6,:)+GS(2,:))
    GS(nx-1,:) = a1*GS(nx-1,:) + a2*(GS(nx-2,:)+GS(nx,:)) + a3*(GS(nx-3,:)+GS(1,:)) &
    	+ a4*(GS(nx-4,:)+GS(2,:)) + a5*(GS(nx-5,:)+GS(3,:))
    GS(nx,:) = a1*GS(nx,:) + a2*(GS(nx-1,:)+GS(1,:)) + a3*(GS(nx-2,:)+GS(2,:)) &
    	+ a4*(GS(nx-3,:)+GS(3,:)) + a5*(GS(nx-3,:)+GS(4,:))
end if



END if


!	Y-direction guassian filter
if (size(y) >= 9) then
DO i=5, size(y)-4
  GS(:,i) = a1*GS(:,i) + a2*(GS(:,i-1)+GS(:,i+1)) + a3*(GS(:,i-2)+GS(:,i+2)) &
    	+ a4*(GS(:,i-3)+GS(:,i+3)) + a5*(GS(:,i-4)+GS(:,i+4))

END DO

if (periody .eq. 1) then
	GS(:,1) = a1*GS(:,1) + a2*(GS(:,ny)+GS(:,2)) + a3*(GS(:,ny-1)+GS(:,3)) &
    	+ a4*(GS(:,ny-2)+GS(:,4)) + a5*(GS(:,ny-3)+GS(:,5))
    GS(:,2) = a1*GS(:,2) + a2*(GS(:,1)+GS(:,3)) + a3*(GS(:,ny)+GS(:,4)) &
    	+ a4*(GS(:,ny-1)+GS(:,5)) + a5*(GS(:,ny-2)+GS(:,6))
    GS(:,3) = a1*GS(:,3) + a2*(GS(:,2)+GS(:,4)) + a3*(GS(:,1)+GS(:,5)) &
    	+ a4*(GS(:,ny)+GS(:,6)) + a5*(GS(:,ny-1)+GS(:,7))
    GS(:,4) = a1*GS(:,4) + a2*(GS(:,3)+GS(:,5)) + a3*(GS(:,2)+GS(:,6)) &
    	+ a4*(GS(:,1)+GS(:,7)) + a5*(GS(:,ny)+GS(:,8))
    

	GS(:,ny-3) = a1*GS(:,ny-3) + a2*(GS(:,ny-4)+GS(:,ny-2)) + a3*(GS(:,ny-5)+GS(:,ny-1)) &
    	+ a4*(GS(:,ny-6)+GS(:,ny)) + a5*(GS(:,ny-7)+GS(:,1))
    GS(:,ny-2) = a1*GS(:,ny-2) + a2*(GS(:,ny-3)+GS(:,ny-1)) + a3*(GS(:,ny-4)+GS(:,ny)) &
    	+ a4*(GS(:,ny-5)+GS(:,1)) + a5*(GS(:,ny-6)+GS(:,2))
    GS(:,ny-1) = a1*GS(:,ny-1) + a2*(GS(:,ny-2)+GS(:,ny)) + a3*(GS(:,ny-3)+GS(:,1)) &
    	+ a4*(GS(:,ny-4)+GS(:,2)) + a5*(GS(:,ny-5)+GS(:,3))
    GS(:,ny) = a1*GS(:,ny) + a2*(GS(:,ny-1)+GS(:,1)) + a3*(GS(:,ny-2)+GS(:,2)) &
    	+ a4*(GS(:,ny-3)+GS(:,3)) + a5*(GS(:,ny-3)+GS(:,4))
end if

END if


END SUBROUTINE



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE gauss_E(GS)
USE globals
USE inputs, ONLY: nx,ny,periodx,periody
!	Apply gaussian filter as define by Cook, A.W., Phys. Fluid 19 (2007)
DOUBLE PRECISION, DIMENSION(nx,ny) :: GS 
DOUBLE PRECISION :: a1,a2,a3,a4,a5


!	Filter Coefficients
a1 = DBLE(3565.0D0/10368.0D0)
a2 = DBLE(3091.0D0/12960.0D0)
a3 = DBLE(1997.0D0/25920.0D0)
a4 = DBLE(149.0D0/12960.0D0)
a5 = DBLE(107.0D0/103680.0D0)

!	X-direction guassian filter
if (size(x) >= 9) then
DO i=5, size(x)-4

    GS(i,:) = a1*GS(i,:) + a2*(GS(i-1,:)+GS(i+1,:)) + a3*(GS(i-2,:)+GS(i+2,:)) &
    	+ a4*(GS(i-3,:)+GS(i+3,:)) + a5*(GS(i-4,:)+GS(i+4,:))
END DO

if (periodx .eq. 1) then
	GS(1,:) = a1*GS(1,:) + a2*(GS(nx,:)+GS(2,:)) + a3*(GS(nx-1,:)+GS(3,:)) &
    	+ a4*(GS(nx-2,:)+GS(4,:)) + a5*(GS(nx-3,:)+GS(5,:))
    GS(2,:) = a1*GS(2,:) + a2*(GS(1,:)+GS(3,:)) + a3*(GS(nx,:)+GS(4,:)) &
    	+ a4*(GS(nx-1,:)+GS(5,:)) + a5*(GS(nx-2,:)+GS(6,:))
    GS(3,:) = a1*GS(3,:) + a2*(GS(2,:)+GS(4,:)) + a3*(GS(1,:)+GS(5,:)) &
    	+ a4*(GS(nx,:)+GS(6,:)) + a5*(GS(nx-1,:)+GS(7,:))
    GS(4,:) = a1*GS(4,:) + a2*(GS(3,:)+GS(5,:)) + a3*(GS(2,:)+GS(6,:)) &
    	+ a4*(GS(1,:)+GS(7,:)) + a5*(GS(nx,:)+GS(8,:))
    

	GS(nx-3,:) = a1*GS(nx-3,:) + a2*(GS(nx-4,:)+GS(nx-2,:)) + a3*(GS(nx-5,:)+GS(nx-1,:)) &
    	+ a4*(GS(nx-6,:)+GS(nx,:)) + a5*(GS(nx-7,:)+GS(1,:))
    GS(nx-2,:) = a1*GS(nx-2,:) + a2*(GS(nx-3,:)+GS(nx-1,:)) + a3*(GS(nx-4,:)+GS(nx,:)) &
    	+ a4*(GS(nx-5,:)+GS(1,:)) + a5*(GS(nx-6,:)+GS(2,:))
    GS(nx-1,:) = a1*GS(nx-1,:) + a2*(GS(nx-2,:)+GS(nx,:)) + a3*(GS(nx-3,:)+GS(1,:)) &
    	+ a4*(GS(nx-4,:)+GS(2,:)) + a5*(GS(nx-5,:)+GS(3,:))
    GS(nx,:) = a1*GS(nx,:) + a2*(GS(nx-1,:)+GS(1,:)) + a3*(GS(nx-2,:)+GS(2,:)) &
    	+ a4*(GS(nx-3,:)+GS(3,:)) + a5*(GS(nx-3,:)+GS(4,:))
end if



END if




END SUBROUTINE
  
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE gauss_N(GS)
USE globals
USE inputs, ONLY: nx,ny,periodx,periody
!	Apply gaussian filter as define by Cook, A.W., Phys. Fluid 19 (2007)
DOUBLE PRECISION, DIMENSION(nx,ny) :: GS 
DOUBLE PRECISION :: a1,a2,a3,a4,a5


!	Filter Coefficients
a1 = DBLE(3565.0D0/10368.0D0)
a2 = DBLE(3091.0D0/12960.0D0)
a3 = DBLE(1997.0D0/25920.0D0)
a4 = DBLE(149.0D0/12960.0D0)
a5 = DBLE(107.0D0/103680.0D0)



!	Y-direction guassian filter
if (size(y) >= 9) then
DO i=5, size(y)-4
  GS(:,i) = a1*GS(:,i) + a2*(GS(:,i-1)+GS(:,i+1)) + a3*(GS(:,i-2)+GS(:,i+2)) &
    	+ a4*(GS(:,i-3)+GS(:,i+3)) + a5*(GS(:,i-4)+GS(:,i+4))

END DO

if (periody .eq. 1) then
	GS(:,1) = a1*GS(:,1) + a2*(GS(:,ny)+GS(:,2)) + a3*(GS(:,ny-1)+GS(:,3)) &
    	+ a4*(GS(:,ny-2)+GS(:,4)) + a5*(GS(:,ny-3)+GS(:,5))
    GS(:,2) = a1*GS(:,2) + a2*(GS(:,1)+GS(:,3)) + a3*(GS(:,ny)+GS(:,4)) &
    	+ a4*(GS(:,ny-1)+GS(:,5)) + a5*(GS(:,ny-2)+GS(:,6))
    GS(:,3) = a1*GS(:,3) + a2*(GS(:,2)+GS(:,4)) + a3*(GS(:,1)+GS(:,5)) &
    	+ a4*(GS(:,ny)+GS(:,6)) + a5*(GS(:,ny-1)+GS(:,7))
    GS(:,4) = a1*GS(:,4) + a2*(GS(:,3)+GS(:,5)) + a3*(GS(:,2)+GS(:,6)) &
    	+ a4*(GS(:,1)+GS(:,7)) + a5*(GS(:,ny)+GS(:,8))
    

	GS(:,ny-3) = a1*GS(:,ny-3) + a2*(GS(:,ny-4)+GS(:,ny-2)) + a3*(GS(:,ny-5)+GS(:,ny-1)) &
    	+ a4*(GS(:,ny-6)+GS(:,ny)) + a5*(GS(:,ny-7)+GS(:,1))
    GS(:,ny-2) = a1*GS(:,ny-2) + a2*(GS(:,ny-3)+GS(:,ny-1)) + a3*(GS(:,ny-4)+GS(:,ny)) &
    	+ a4*(GS(:,ny-5)+GS(:,1)) + a5*(GS(:,ny-6)+GS(:,2))
    GS(:,ny-1) = a1*GS(:,ny-1) + a2*(GS(:,ny-2)+GS(:,ny)) + a3*(GS(:,ny-3)+GS(:,1)) &
    	+ a4*(GS(:,ny-4)+GS(:,2)) + a5*(GS(:,ny-5)+GS(:,3))
    GS(:,ny) = a1*GS(:,ny) + a2*(GS(:,ny-1)+GS(:,1)) + a3*(GS(:,ny-2)+GS(:,2)) &
    	+ a4*(GS(:,ny-3)+GS(:,3)) + a5*(GS(:,ny-3)+GS(:,4))
end if

END if


END SUBROUTINE
