!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.

!	Subroutines relating to the initialization
!	of different types of problems, using the
!	"case" construct.  
!
!	prob_init- called in main.f90 and used to specify the problem
!		parameters.  This routine in term calls three common routines
!		and one problem specific routine
!
!	input- reads in the input file for each problem
!
!	alloc_grid- with prob specific info known, now initialize
!		arrays as specified
!
!	grid_gen- simple routine to generate the cartesian mesh
!
!	restart- routine for reading in RT restart data
!
!	Note: Problem specific routines are found in prob.f90


SUBROUTINE prob_init
  USE globals
  USE inputs
  IMPLICIT NONE  
  INTEGER :: in_unit
  DOUBLE PRECISION :: dum,dum1
  
  bx1 = 0
  bxn = 0
  by1 = 0
  byn = 0
  
  CALL alloc_grid
  CALL grid_gen
  
  if (restart_log .eq. 1) then
     CALL restart
  else 
     CALL initialize 
     t_init = zero
     counter = 0
  end if
  
  Grav(:,:,1) = Gx
  Grav(:,:,2) = Gy
  


END SUBROUTINE prob_init

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


SUBROUTINE alloc_grid
  USE globals
  USE inputs, ONLY: nx,ny,ns,tt
  !USE AFLES_vars
  IMPLICIT NONE

  ALLOCATE(x(nx))
  ALLOCATE(y(ny))
  ALLOCATE(rho(nx,ny))
  ALLOCATE(p(nx,ny))
  ALLOCATE(T(nx,ny))
  ALLOCATE(u(nx,ny))
  ALLOCATE(v(nx,ny))
  ALLOCATE(e(nx,ny))
  ALLOCATE(Grav(nx,ny,2))
  ALLOCATE(ss(nx,ny))
  ALLOCATE(ie(nx,ny))
  ALLOCATE(hi(nx,ny,ns))
  ALLOCATE(Rgas(nx,ny))
  ALLOCATE(gamma(nx,ny))
  ALLOCATE(tauxx(nx,ny))
  ALLOCATE(tauxy(nx,ny))
  ALLOCATE(tauyy(nx,ny))
  ALLOCATE(dQx(nx,ny))
  ALLOCATE(dQy(nx,ny))
  ALLOCATE(Q(nx,ny,4+ns-1))
  ALLOCATE(G(nx,ny,4+ns-1))
  ALLOCATE(F(nx,ny,4+ns-1))
  ALLOCATE(source(nx,ny,4+ns-1))
  ALLOCATE(Jx(nx,ny,ns))
  ALLOCATE(Jy(nx,ny,ns))
  ALLOCATE(YY(nx,ny,ns))
  
  ALLOCATE(dudx(nx,ny))
  ALLOCATE(dudy(nx,ny))
  ALLOCATE(dvdx(nx,ny))
  ALLOCATE(dvdy(nx,ny))
  ALLOCATE(Tx(nx,ny))
  ALLOCATE(Ty(nx,ny))
  ALLOCATE(mu(nx,ny))
  ALLOCATE(beta(nx,ny))
  ALLOCATE(kappa(nx,ny))
  ALLOCATE(diff(nx,ny,ns))
  ALLOCATE(diff_f(ns))
  
  
END SUBROUTINE alloc_grid

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE grid_gen
USE globals
USE inputs
  !	Grid spacing - X-direction
  DO i = 1, nx
    x(i) = dx*DBLE(i-1)	
  END DO
  
  !	Grid spacing - Y-direction
  DO i = 1, ny
    y(i) = dy*DBLE(i-1)      
  END DO
  
  x = x + x1
  y = y + y1

  print*,x(1)
END SUBROUTINE


SUBROUTINE restart
  USE GLOBALS
  USE inputs
  !USE AFLES_vars
  !USE metrics
  IMPLICIT NONE
  INTEGER :: re_unit,species
  character*80 comments,filename
  re_unit = 55
  
  
  !	Get restart file name
  ! Read restart meta-file
  OPEN(2,file='restart.dat',STATUS='OLD')
  READ(2,*) counter,t_init,filename
  CLOSE(2)

  OPEN(UNIT=re_unit,FILE=filename,STATUS='OLD')
  READ(re_unit,*) comments
  READ(re_unit,*) comments

  DO J=1,ny
     DO I = 1,nx
        READ(re_unit,*) x(I),y(J),p(I,J), rho(I,J), T(I,J),u(I,J),v(I,J),YY(I,J,1)
     END DO
  END DO
  CLOSE(re_unit)
  
  YY(:,:,2) = one - YY(:,:,1)

  !	Mixed gamma value here
  gamma = zero
  DO species = 1, ns
     gamma = gamma + YY(:,:,species)/gamma_f(species)
  END DO
  gamma = one/gamma
  
  e = p/(gamma-one) + rho*(one/two)*(u*u + v*v)
  
  Q(:,:,1) = rho*YY(:,:,1)
  Q(:,:,2) = rho*u
  Q(:,:,3) = rho*v
  Q(:,:,4) = e
  
  DO i=1,ns-1
     Q(:,:,i+4) = rho*YY(:,:,i+1)
  END DO
  

  CALL flow_vars

  CALL prob_restart

  
END SUBROUTINE restart
