!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.


!	Routine containing the problem 
!	initializations for different cases
!	called by init.f90 (prob_init)
!
!	boundary- problem specific bc's called here and assign in 
!		appropriate case
MODULE RT_data
  SAVE
  ! ---------------------------------------------------------------------------
  ! Place Problem dependent state data here, similar to inputs.f and globals.f
  ! ---------------------------------------------------------------------------
  CHARACTER(LEN=30) 	:: prob_jobname = 'RT' 
  DOUBLE PRECISION      :: band = 4.0D0
  DOUBLE PRECISION      :: pertpeak = 9.0D0
  DOUBLE PRECISION      :: pertrms = 1.5D0
  DOUBLE PRECISION      :: thick = 3.0D0
  DOUBLE PRECISION      :: ReW = 100.0D0
  DOUBLE PRECISION      :: Sch = 1.0D0
  DOUBLE PRECISION      :: Pr = 1.0D0
  DOUBLE PRECISION      :: StN = 1.0D0

  DOUBLE PRECISION      :: At = 0.5D0
  DOUBLE PRECISION      :: lam = 1.0D0
  DOUBLE PRECISION      :: Ptop = 1.0D5
  DOUBLE PRECISION      :: Pbot = 1.0D5

END MODULE RT_data

! -----------------------------------------------------------------------------
! prob_inputs
! Read problem-dependent inputs and save in module variables
! must also define 'jobname'
! Called by the Miranda main program.
! -----------------------------------------------------------------------------

SUBROUTINE prob_inputs(fileName)
  USE RT_data
  USE globals, ONLY : inputUnit,jobname,R,two,pi
  USE inputs
  IMPLICIT NONE
  CHARACTER(LEN=*), INTENT(IN) :: fileName
  DOUBLE PRECISION :: Pmid,R1,R2

  ! Uncomment to define a namelist
  NAMELIST /RT_vars/ prob_jobname,band,pertpeak,pertrms,thick,ReW,Sch,Pr,StN 

  ! Uncomment to open and read a namelist file
  OPEN(UNIT=inputUnit,FILE=TRIM(fileName),FORM='FORMATTED',STATUS='OLD')
  READ(UNIT=inputUnit,NML=RT_vars)
  CLOSE(inputUnit)
  
  ! give the right name
  jobname = TRIM(prob_jobname)


  ! Make domain always 2*pi wide and set the dx & dy accordingly
  dx = two*pi / DBLE(nx-1)
  dy = dx
  periodx = 1
  periody = 0
  
  ! Always put interface in the middle
  x1 = 0.0D0
  y1 = -DBLE(ny)*dy / two


  ! Get relevant lenght scale and set gravity based on Stratification Number StN
  At = (molwt(2)-molwt(1))/(molwt(2)+molwt(1))
  lam = DBLE(nx)*dx / pertpeak
  Gy = - StN * R*T0 / (At * lam)


  ! Get the bottom and top pressures
  Ptop = P0 
  R2 = R/molwt(2)
  R1 = R/molwt(1)
  Pmid = P0 * exp( Gy / (R2*T0 ) * y1 )
  Pbot =Pmid* exp( Gy / (R1*T0 ) * y1 )
  
  print*,Ptop,Pbot

  ! Non-dimensionalize the viz time and sim times
  viztime = viztime/(sqrt(At*abs(Gy)/lam))
  simtime = simtime/(sqrt(At*abs(Gy)/lam))


END SUBROUTINE prob_inputs



SUBROUTINE initialize
USE globals
USE inputs
USE RT_data
IMPLICIT NONE
DOUBLE PRECISION, DIMENSION(nx,ny) :: tmp,dum
DOUBLE PRECISION :: eps,rho0,cv,cp
INTEGER :: ww,wy,species

!  Initialization routine for RT case
CALL RTinit

!	Mixed gamma value here
gamma = zero
DO species = 1, ns
	gamma = gamma + YY(:,:,species)/gamma_f(species)
END DO
gamma = one/gamma

e = p/(gamma-one) + rho*(one/two)*(u*u + v*v)
Q(:,:,1) = rho*YY(:,:,1)
Q(:,:,2) = rho*u
Q(:,:,3) = rho*v
Q(:,:,4) = e

DO i=1,ns-1
	Q(:,:,i+4) = rho*YY(:,:,i+1)
END DO

CALL flow_vars



!! Set the viscous terms here
rho0 = sum(rho)/(dble(nx*ny))
mu_f = sqrt(At*abs(Gy)*lam)
mu_f = mu_f * lam * rho0 / ReW

diff_f = mu_f / (rho0 * Sch)


cv = R/(gamma_f(1)-1.0D0)/ ( molwt(1)+molwt(2))*2
cp = gamma_f(1)*cv
kappa_f = cp * mu_f / Pr


beta_f = 0.0D0


END SUBROUTINE 


SUBROUTINE RTinit
  USE GLOBALS 
  USE inputs
  USE RT_data
  IMPLICIT NONE
  DOUBLE PRECISION, DIMENSION(nx,ny) :: tmp,dum,mm
  DOUBLE PRECISION, DIMENSION(nx) :: pert,phase,gss
  DOUBLE PRECISION :: x2,Ptmp,rms,off
  INTEGER, DIMENSION(8) :: dtval
  INTEGER, DIMENSION(1) :: seed
  INTEGER :: species,ii,kk
  
  pertrms = pertrms*dx
  thick = thick*dx
  
  T = T0
  
  DO i=1,nx
     gss(i) = exp( -(dble(i)-pertpeak)/(2.0D0*band**2))
  END DO
  x2 = maxval(gss)
  gss = gss / x2

  CALL time_seed()
  CALL random_number(phase)

  pert = zero  
  DO i=1,nx/2
     pert = pert + pertrms * gss(i) * sin( dble(i) * two*pi*x/( dble(nx-1)*dx) + two*pi*phase(i))
  END DO

  
  !	Hyperbolic tangent function for interface 
  DO i = 1,nx
     DO j = 1,ny
        tmp(i,j) = (y(j) + pert(i))/thick
     END DO
  END DO
  dum = tanh(tmp)
  
  !	Define the mass fraction for the two species
  YY(:,:,1) = (one/two)*(one - dum)
  YY(:,:,2) = one - YY(:,:,1)
  

  DO i = 1,nx
     DO j = 1,ny
        P(i,j) = P0*exp(one/thick*(y(j)+pert(i))) &
             **(molwt(1)*Gy/(R*T(i,j)*one/thick))*(exp(two*one/thick*(y(j)+pert(i))) &
             *molwt(1)+molwt(2))**(Gy*(molwt(2)-molwt(1))/(two*R*T(i,j)*one/thick)) & 
             /(exp(one/thick*(((ny-1)*dy+y1)+pert(i)))**(molwt(1)*Gy/(R*T(i,j)*one/thick))* &
             (exp(two*one/thick*(((ny-1)*dy+y1)+pert(i)))*molwt(1)+molwt(2))**(Gy* &
             (molwt(2)-molwt(1))/(two*R*T(i,j)*one/thick)))
     END DO
  END DO
  
  T = T0
  u = zero
  v = zero
  
  !	Mixed gamma value here
  gamma = zero
  DO species = 1, ns
     gamma = gamma + YY(:,:,species)/gamma_f(species)
  END DO
  gamma = one/gamma
  
  
  !	Specific Gas Constant for gases
  Rgas = zero
  DO i = 1, ns
     Rgas = Rgas + YY(:,:,i)/molwt(i)
  END DO
  Rgas = R*Rgas 
  
  rho = p/(Rgas*T)
  e = p/(gamma-one) + rho*(one/two)*(u*u + v*v)
  
  Q(:,:,1) = rho*YY(:,:,1)
  Q(:,:,2) = rho*u
  Q(:,:,3) = rho*v
  Q(:,:,4) = e
  
  DO i=1,ns-1
     Q(:,:,i+4) = rho*YY(:,:,i+1)
  END DO
  

  CALL flow_vars


END SUBROUTINE RTinit


SUBROUTINE boundary
  USE inputs
  USE globals
  USE RT_data
  IMPLICIT NONE
  INTEGER :: nn
  

  u(:,1:3) = zero
  u(:,ny-2:ny) = zero
  v(:,1:3) = zero
  v(:,ny-2:ny) = zero

  T(:,1)  = T0
  T(:,ny) = T0
  !p(:,1)  = 2.0D0*p(:,2)   - p(:,3)
  !p(:,ny) = 2.0D0*p(:,ny-1)- p(:,ny-2)
  !p(:,1)  = Pbot
  !p(:,ny) = Ptop

  
  
  !! Bounds on species 
  !WHERE(YY .GT. one)
  !   YY = one
  !END WHERE
  !WHERE(YY .LT. zero)
  !   YY = zero
  !END WHERE
  
  !DO nn=1,ns
  !   !CALL gauss(YY(:,:,nn))
  !   CALL compact_filter(YY(:,:,nn),1,nx,ny,periodx)
  !   CALL compact_filter(YY(:,:,nn),2,nx,ny,periody)
  !END DO

  
  !	Specific Gas Constant for gases
  Rgas = zero
  DO nn = 1, ns
     Rgas = Rgas + YY(:,:,nn)/molwt(nn)
  END DO
  Rgas = R*Rgas 

  P = rho *  (Rgas * T)
  e = p/(gamma-one) + rho*(one/two)*(u*u + v*v)

  Q(:,:,1) = rho*YY(:,:,1)
  Q(:,:,2) = rho*u
  Q(:,:,3) = rho*v
  Q(:,:,4) = e
  Q(:,:,5) = rho*YY(:,:,2)



END SUBROUTINE boundary


SUBROUTINE prob_restart
  USE globals, ONLY: mu_f,beta_f,kappa_f,diff_f,R,rho
  USE inputs, ONLY: gamma_f,molwt,Gy,nx,ny,dx
  USE RT_data, ONLY: pertpeak,ReW,Pr,Sch,At,lam
  IMPLICIT NONE
  DOUBLE PRECISION :: eps,rho0,cv,cp
  
  ! Put stuff here for prob dependent restart
  
  !! Set the viscous terms here
  rho0 = sum(rho)/(dble(nx*ny))
  mu_f = sqrt(At*abs(Gy)*lam)
  mu_f = mu_f * lam * rho0 / ReW
  
  diff_f = mu_f / (rho0 * Sch)
  
  
  cv = R/(gamma_f(1)-1.0D0)/ ( molwt(1)+molwt(2))*2
  cp = gamma_f(1)*cv
  kappa_f = cp * mu_f / Pr
  
  
  beta_f = 0.0D0
  
  
END SUBROUTINE prob_restart



!! PRN Generator seeder based on system clock
SUBROUTINE time_seed()
  IMPLICIT NONE
  INTEGER :: i, n, clock
  INTEGER, DIMENSION(:), ALLOCATABLE :: seed
  INTEGER, DIMENSION(8) :: dt
  
  CALL RANDOM_SEED(size = n)
  ALLOCATE(seed(n))
  CALL SYSTEM_CLOCK(COUNT=clock)
  CALL date_and_time(values=dt)
  clock = dt(8)

  i=1
  seed = clock + 71 * (/ (i - 1, i = 1, n) /)
  CALL RANDOM_SEED(PUT = seed)
  DEALLOCATE(seed)

END SUBROUTINE time_seed
