!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.


!	Main subroutine for advancing the time via a 
!	5 step, 4th order Runge-Kutta (RK4) method
!	Subroutine pertaining to each step are also here.
!
!	RK4- main routine that updates the conserved variables (Q)
!		for 5 sub-iterations (loop) and then ends
!
!	get_FLUX- used to re-calculate the flux variables, or gradient
!		at each sub-iteration.
!	
!	sub_filter- 8th order, compact filter (pentadiagonal) used to 
!		filter the conserved variables at every sub-iteration.
!
!	rhs- computes the rhs (before the gradient is taken) of the 
!		equation.  Straight forward as defined in the NS eqs. and 
!		calls the routine, get_tau (found in solver) to compute the 
!		viscous stresses, heat flux and mass diffusion terms.

SUBROUTINE RK4
USE globals
USE inputs
IMPLICIT NONE
DOUBLE PRECISION, DIMENSION(5) :: Ark, Brk, eta
DOUBLE PRECISION, DIMENSION(size(Q,1),size(Q,2),size(Q,3)) :: FLUX,PHI,tmp1,tmp2
INTEGER :: ii

Ark(1) = zero
Ark(2) = -6234157559845D0/12983515589748D0
Ark(3) = -6194124222391D0/4410992767914D0
Ark(4) = -31623096876824D0/15682348800105D0
Ark(5) = -12251185447671D0/11596622555746D0

Brk(1) = 494393426753D0/4806282396855D0
Brk(2) = 4047970641027D0/5463924506627D0
Brk(3) = 9795748752853D0/13190207949281D0
Brk(4) = 4009051133189D0/8539092990294D0
Brk(5) = 1348533437543D0/7166442652324D0
 
!eta(1) = 494393426753/4806282396855
!eta(2) = 4702696611523/9636871101405
!eta(3) = 3614488396635/5249666457482
!eta(4) = 9766892798963/10823461281321
!eta(5) = one

!	Initialize some intermediate arrays
tmp1 = zero
tmp2 = zero
PHI = zero

!	Get primative flow variables
CALL flow_vars

DO ii=1,5
  	
    CALL rhs
    CALL get_FLUX(FLUX)
    
    tmp1 =  Ark(ii)*PHI
    PHI = -dt*FLUX + tmp1

    tmp2 =  Brk(ii)*PHI
	Q =  Q + tmp2 

    !if (k > filter_lim) then
    CALL sub_filter(Q)
	!print*,k,ii
    !end if

    CALL flow_vars
    CALL boundary
 
END DO

!	Euler Explicit 
!Q = Q - dt*FLUX
  

END SUBROUTINE RK4



SUBROUTINE get_FLUX(FLUX)
USE globals , only: i,F,G,zero,source
USE inputs, ONLY: nx,ny,ns
!USE metrics, only: dxy
DOUBLE PRECISION, DIMENSION(nx,ny,size(F,3)) :: FLUX, qx,qy


!	Take the divergence of F and G
CALL div(G,F,qx,qy,4+(ns-1))


!	Get the rest of the RHS.  Gravity is only non-differentiated term.
CALL gravity

!	Total flux
FLUX = qx + qy 

DO i=1,4+(ns-1)
	FLUX(:,:,i) = FLUX(:,:,i) !/dxy
END DO	

FLUX = FLUX - source


END SUBROUTINE get_FLUX


SUBROUTINE sub_filter(Qf)
USE inputs, only: periodx,periody,nx,ny,ns
IMPLICIT NONE
DOUBLE PRECISION, DIMENSION(nx,ny,4+(ns-1)) :: Qf
INTEGER :: i

!	Filter after each step
!	filter in the x-direction
DO i = 1, 4+(ns-1)
	CALL compact_filter(Qf(:,:,i),1,size(Qf,1),size(Qf,2),periodx)
END DO

!	filter in the y-direction
DO i = 1, 4+(ns-1)
	CALL compact_filter(Qf(:,:,i),2,size(Qf,1),size(Qf,2),periody)
END DO

END SUBROUTINE sub_filter


SUBROUTINE rhs
USE globals
USE inputs, ONLY: ns
DOUBLE PRECISION :: Tfloor
INTEGER :: species

!	Get the viscous terms and heat flux
CALL get_tau

G(:,:,1) = rho*u*YY(:,:,1) + Jx(:,:,1)
G(:,:,2) = rho*u*u + p - tauxx
G(:,:,3) = rho*u*v - tauxy
G(:,:,4) = (e+p)*u - tauxx*u - tauxy*v + dQx

F(:,:,1) = rho*v*YY(:,:,1) + Jy(:,:,1)
F(:,:,2) = rho*u*v - tauxy
F(:,:,3) = rho*v*v + p - tauyy
F(:,:,4) = (e+p)*v - tauxy*u -tauyy*v + dQy

DO species = 1, ns-1
	G(:,:, 4+(species) ) = rho*u*YY(:,:,species+1) + Jx(:,:,species+1)
	F(:,:, 4+(species) ) = rho*v*YY(:,:,species+1) + Jy(:,:,species+1)
END DO


END SUBROUTINE rhs
