!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.


!	solver.f90- Routines for the overall solution typically called
!	at every time step
!	
!	flow_vars- Reads in the conserved variables and computes the
!		physical flow variables based on the ideal gas law.
!	
!	gravity- applies gravity to source term, acting basically like
!		a body force
!	
!	get_tau- get the viscous stresses, heat flux and the diffusion. 
!		Used in rk4.f90 (get_rhs) at every sub-iteration.  Calls the 
!		AFLES variable routine found below
!
!	afles_visc/afles_con/afles_diff- AFLES terms to compute the artificial
!		fluid properties of the flow	
!


SUBROUTINE flow_vars
USE globals
USE inputs
DOUBLE PRECISION :: Tfloor
INTEGER :: species

!	Calculate the actual flow variables at each time step

!	Primative variables
rho = Q(:,:,1)
DO species = 1, ns-1
	rho = rho + Q(:,:,4+(species) )
END DO

u = Q(:,:,2)/rho
v = Q(:,:,3)/rho
e = Q(:,:,4)

!	Mass Fractions
YY(:,:,1) = Q(:,:,1)/rho
DO species = 1,ns-1
	YY(:,:,species+1) = Q(:,:, 4+(species) )/rho
END DO

!	Specific Gas Constant for gases
Rgas = zero
DO species = 1, ns
	Rgas = Rgas + YY(:,:,species)/molwt(species)
END DO
Rgas = R*Rgas 

!   Mass weighted gamma values (based on Partial Pressure)
gamma = zero
DO species = 1, ns
	gamma = gamma + YY(:,:,species)/molwt(species)/(gamma_f(species)-one)
END DO
gamma = one/gamma*Rgas/R + one


!	Pressure Field
P = (gamma-one)*(e-(one/two)*rho*(u*u+v*v))
where (P <= zero) 
	P = zero
end where


!	Temperature
T = P/(Rgas*rho)
Tfloor = 1.0D-1*minval(abs(T))
if (Tfloor .eq. 0) then
	Tfloor = maxval(T)*1.0D-3
end if
where (T <= Tfloor)
	T = Tfloor
end where


!	Internal Energy and Sound Speed
ie = Rgas*T/(gamma-one)
ss = sqrt(abs(gamma*Rgas*T))


END SUBROUTINE flow_vars


SUBROUTINE gravity
USE GLOBALS
USE INPUTS
IMPLICIT NONE
!	Include the source terms into the 
!	Equation, including the gravity

!	Source due to gravity
source = zero						!	continuity for all species
source(:,:,2) = rho*Grav(:,:,1)				!	X-momentum
source(:,:,3) = rho*Grav(:,:,2)				!	Y-momentum
source(:,:,4) = rho*(Grav(:,:,1)*u + Grav(:,:,2)*v)	!	Energy


END SUBROUTINE gravity

!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE get_tau
USE GLOBALS
USE INPUTS
IMPLICIT NONE
DOUBLE PRECISION, DIMENSION(nx,ny,ns) :: dYdx,dYdy
DOUBLE PRECISION, DIMENSION(nx,ny) :: tmpx,tmpy
INTEGER :: species

tmpx = zero
tmpy = zero


!	Subroutine to calculate the viscous stress tensor tau
CALL grad(u,dudx,dudy)
CALL grad(v,dvdx,dvdy)
CALL grad(T,Tx,Ty)

mu = mu_f
beta = beta_f
kappa = kappa_f


!	Viscous stress
tauxx = two*mu*dudx + ( beta - (two/three)*mu )*(dudx + dvdy)
tauxy = mu*(dudy + dvdx)
tauyy = two*mu*dvdy + ( beta - (two/three)*mu )*(dudx + dvdy)

!	Mass diffusion 
IF (ns > 1) THEN
  DO i = 1, ns
	CALL grad(YY(:,:,i),dYdx(:,:,i),dYdy(:,:,i))
  END DO

  DO i = 1, ns 
    diff(:,:,i) = diff_f(i) 
    DO j = 1, ns
	tmpx = tmpx + diff(:,:,i)*dYdx(:,:,i)
        tmpy = tmpy + diff(:,:,i)*dYdy(:,:,i)
    END DO
    Jx(:,:,i) = -rho*(diff(:,:,i)*dYdx(:,:,i) - YY(:,:,i)*tmpx)
    Jy(:,:,i) = -rho*(diff(:,:,i)*dYdy(:,:,i) - YY(:,:,i)*tmpy)
  END DO
ELSE
  Jx=zero
  Jy=zero
END IF

! Heat conduction
dQx = -kappa*Tx 
dQy = -kappa*Ty 


END SUBROUTINE get_tau
