!	Britton J. Olson
!	Stanford University- Department of Aero/Astro
!	miranda_jr 2-D LES code
!	Solves the compressible Navier Stokes equations
!	in 2-D for arbitrary number of fluids
!	10th order compact scheme as defined by Lele
!	4th order, 5 step RK4 for temporal integration 

!    Copyright (C) 2011  Britton Olson
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License at http://www.gnu.org/licenses/
!    for more details.




!	Subroutines to select the appropriate dt for 
!	stability and for viz-dumps
!	get_dt - adjusts the dt (only decreases) in order
!		to output viz_files as specified
!	courant - ensures stability for each dt considering
!		convection, and all the AFLES terms


SUBROUTINE get_dt(viz,output)
USE globals
USE inputs
INTEGER :: viz
INTEGER :: output

!	Get the stability defined dt
CALL flow_vars
CALL rhs
CALL boundary
CALL courant

!	Modify dt to achieve specified output frequency
!	Also signal output flag
if ( (time + dt) > (viztime + time_v ) ) then
  dt = ( viztime+time_v ) - time
  viz = viz + 1
  output = 1
else
  output = 0
end if

if (k <= 2) then
  dt = .1*dt
end if


END SUBROUTINE get_dt


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


SUBROUTINE courant
  USE globals
  USE inputs
  IMPLICIT NONE
  DOUBLE PRECISION :: dt_conv, dt_mu, dt_beta, dt_kappa, dt_diff, safe, del
  INTEGER :: ramp,order
  

  !	CFL condition 
  dt = MAXVAL( abs(u/dx) + abs(v/dy) + ss*sqrt(one/dx**2 + one/dy**2) )
  
  dt_conv = cfl/dt
  
  del = sqrt(dx*dy)
  
  !	AFLES terms
  dt_mu = maxval(mu/(rho*del**2))
  dt_beta = maxval(beta/(rho*del**2))
  dt_kappa = maxval(kappa*T/(rho*(ss**2)*del**2))
  
  if (ns > 1) then
     dt_diff = maxval(diff/del**2)
     if (dt_diff <= zero) then
    	dt_diff = 10.0D0*dt_conv
     else
        dt_diff = 0.2D0*one/dt_diff
     end if
  else
     dt_diff = two*dt_conv
  end if

  if (dt_mu <= zero) then
     dt_mu = 10.0D0*dt_conv
  else
     dt_mu = 0.2D0*one/dt_mu
  end if

  if (dt_beta <= zero) then
     dt_beta = 10.0D0*dt_conv
  else
     dt_beta = 0.1D0*one/dt_beta
  end if
  if (dt_kappa <= zero) then
     dt_kappa = 10.0D0*dt_conv
  else
     dt_kappa = 0.2D0*one/dt_kappa
  end if
  
  !	Overall min dt
  dt = min(dt_conv,dt_mu,dt_beta,dt_kappa,dt_diff)
  
  if (dt .eq. dt_mu) then
     stab = "Shear"
  elseif (dt .eq. dt_beta) then
     stab = "Bulk"
  elseif (dt .eq. dt_kappa) then
     stab = "Cond"
  elseif (dt .eq. dt_diff) then
     stab = "Diff"
  else
     stab = "Conv"
  end if
  
  !	Ramp for the first n steps
  ramp = 30	!  Number of steps in ramp
  order = 1	!  Order of the ramp
  if (k <= ramp) then 
     safe = DBLE(k)**order/ (DBLE(ramp)**order)
     stab = 'Ramp'
  else
     safe = one
     
  end if
  
  dt = dt*safe
  
END SUBROUTINE courant
